from pydantic import BaseModel
from xml.etree import ElementTree as ET
from enum import Enum

class RequestAction(Enum):
    TRACK = "Track"
    
class RequestOption(Enum):
    ACTIVITY = "activity"
    NONE = "none"

class TrackRequest(BaseModel):
    track_type: str
    reference_number_or_tracking_number: str
    
    request_action: str = RequestAction.TRACK
    request_option: str = RequestOption.ACTIVITY
    customer_context: str = "Customer Context"
    xpci_version: str = "1.0"
    
    def xml(self) -> ET.Element:
        root = ET.Element("TrackRequest")
        request_el = ET.SubElement(root, "Request")
        transaction_reference_el = ET.SubElement(request_el, "TransactionReference")
        ET.SubElement(transaction_reference_el, "CustomerContext").text = self.customer_context
        ET.SubElement(transaction_reference_el, "XpciVersion").text = self.xpci_version
        ET.SubElement(request_el, "RequestAction").text = self.request_action
        ET.SubElement(request_el, "RequestOption").text = self.request_option
        track_type_el = ET.SubElement(root, self.track_type)
        ET.SubElement(track_type_el, "Value").text = self.reference_number_or_tracking_number
        
        return root
    


from __future__ import annotations
from pydantic import BaseModel
from typing import Optional
from UPS_SDK.models.Address import Address

class ActivityLocation(BaseModel):
    Address: Optional[Address]
    Description: Optional[str]


from __future__ import annotations
from pydantic import BaseModel
from UPS_SDK.models.UnitOfMeasurement import UnitOfMeasurement
class PackageWeight(BaseModel):
    UnitOfMeasurement: UnitOfMeasurement
    Weight: str


from __future__ import annotations
from dataclasses import dataclass


@dataclass
class TrackError(BaseException):
    ErrorSeverity: str
    ErrorCode: str
    ErrorDescription: str
    
    def __repr__(self) -> str:
        return f"""
    
    
        \rError Code: {self.ErrorCode}
        \rDescription: {self.ErrorDescription}
    """
    
    def __str__(self) -> str:
        return f"""
    
    
        \rError Code: {self.ErrorCode}
        \rDescription: {self.ErrorDescription}
    """


from __future__ import annotations
from datetime import datetime
from pydantic import BaseModel, Field
from typing import List, Optional
from UPS_SDK.models.Package import Package
from UPS_SDK.models.ShipTo import ShipTo
from UPS_SDK.models.Shipper import Shipper
from UPS_SDK.models.ReferenceNumberItem import ReferenceNumberItem
from UPS_SDK.models.Service import Service
from UPS_SDK.models.ShipmentWeight import ShipmentWeight

class Shipment(BaseModel):
    Shipper: Shipper
    ShipTo: ShipTo
    ShipmentWeight: Optional[ShipmentWeight]
    Service: Service
    ReferenceNumbers: Optional[List[ReferenceNumberItem]] = Field(None, alias="ReferenceNumber")
    ShipmentIdentificationNumber: str
    PickupDate: str
    ScheduledDeliveryDate: Optional[str]
    Package: Package
    
    @staticmethod
    def delivered_date_to_str(date: datetime, description: str):
        
        pass
    
    @property
    def delivered_date(self) -> Optional[datetime]:
        last_activity = self.Package.AllActivity[0]
        if self.Package.is_delivered:
            return last_activity.get_date
        return None
    
    @property
    def scheduled_delivery_date(self):
        date = None
        if self.Package.RescheduledDeliveryDate is not None :
            date = datetime.strptime(self.Package.RescheduledDeliveryDate, "%Y%m%d")
        elif self.ScheduledDeliveryDate is not None:
            date = datetime.strptime(self.ScheduledDeliveryDate, "%Y%m%d")
        return date
    
    
    
        


from __future__ import annotations
from pydantic import BaseModel
from typing import Optional

class Address(BaseModel):
    City: Optional[str] = None
    StateProvinceCode: Optional[str] = None
    CountryCode: str
    PostalCode: Optional[str] = None
    AddressLine1: Optional[str] = None


from __future__ import annotations
from pydantic import BaseModel
from UPS_SDK.models.Response import Response
from UPS_SDK.models.Shipment import Shipment
from typing import Any

class TrackResponse(BaseModel):
    Response: Response
    Shipment: Shipment
    
    def __init__(__pydantic_self__, **data: Any) -> None:
        if isinstance(data["Shipment"]["Package"]["Activity"], dict):
            data['Shipment']["Package"]["Activity"] = [data["Shipment"]["Package"]["Activity"]]
        if isinstance(data["Shipment"]["ReferenceNumber"], dict):
            data["Shipment"]["ReferenceNumber"] = [data['Shipment']["ReferenceNumber"]]
            
        super().__init__(**data)